import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.util.LinkedList;
import java.util.PriorityQueue;

/**
 * Eine Klasse zur Darstellung eines Graphen.
 */
public class Graph
{
    private int breite, hoehe, anzahl;
    private String farbe;

    /**
     * Erzeuge eine neue Knotenliste mit anzahl vielen Punkten.
     */
    public Graph(int anzahl)
    {
        this.anzahl = anzahl;
        breite = 500;
        hoehe = 500;
        farbe = "hellgrau";
        Kante.setzeDimension(breite, hoehe);
        Knoten.setzeDimension(breite, hoehe);
        for (int i=0;i<anzahl;i++){ new Knoten(); }
    }
    
    /**
     * Erzeuge einen Testgraphen mit 5 Knoten.
     */
    public Graph()
    {
        this.anzahl = 5;
        breite = 300;
        hoehe = 300;
        farbe = "blau";
        Kante.setzeDimension(breite, hoehe);
        Knoten.setzeDimension(breite, hoehe);
        new Knoten(10, 10);
        new Knoten(220, 30);
        new Knoten(60, 110);
        new Knoten(110, 280);
        new Knoten(270, 190);
    }
    
    /**
     * verknuepft alle Knoten zu einem Graphen.
     */
    public void vollstaendig(){
        Kante.loescheKanten();
        for (int i=0;i<anzahl;i++)
        for (int j=i+1;j<anzahl;j++)
        new Kante(Knoten.gibKnoten(i),Knoten.gibKnoten(j));
    }
    
    // **************** Abschnitt fuer minmal spanning tree ******************
    
    /**
     *  Algorithmus von Prim
     */
    public void minimal(){
        LinkedList<Knoten> erreichteKnoten = new LinkedList();
        LinkedList<Kante> baum = new LinkedList();
        vollstaendig();zeichne();
        Leinwand.gibLeinwand().warte(500);
        farbe = "rot";
        // Waehle einen beliebigen Startknoten und jeweils unter den fortsetzenden
        // Kanten die mit der kürzesten Laenge, die keinen Zyklus erzeugt.
        Knoten weiter = Knoten.gibKnoten(0); // weiter ist zunaechst der Startknoten
        erreichteKnoten.add(weiter);
        PriorityQueue<Kante> nachfolgeKanten = new PriorityQueue<Kante>(weiter.gibSortierteKanten());
        Kante aktuelleKante;
        PriorityQueue<Kante> weitereKanten;
        while ((erreichteKnoten.size()<anzahl) && !nachfolgeKanten.isEmpty())
        {       // neue Kante einfuegen
            aktuelleKante = nachfolgeKanten.poll(); // holt die erste Kante und entfernt aus Queue
            if (erreichteKnoten.contains(aktuelleKante.getP1())) weiter = aktuelleKante.getP2();
            else weiter = aktuelleKante.getP1();    // freien Knoten finden
            if (!erreichteKnoten.contains(weiter))  // kein Zyklus
            {       // freien Knoten expandieren
                baum.add(aktuelleKante);
                zeichne(aktuelleKante);
                Leinwand.gibLeinwand().warte(200);
                weitereKanten = new PriorityQueue<Kante>(weiter.gibSortierteKanten());
                for (Kante kante : weitereKanten) // nachfolgekanten aktualisieren
                if (!nachfolgeKanten.contains(kante))
                nachfolgeKanten.add(kante);
                erreichteKnoten.add(weiter);
            }
        }
        // zu entfernende Kanten bestimmen
//         LinkedList<Kante> rausMuessen = new LinkedList();
//         for (int i=0;i<Kante.gibAnzahl();i++)
//         if (!baum.contains(Kante.gibKante(i))) rausMuessen.add(Kante.gibKante(i));
//         for (Kante kante : rausMuessen) kante.entferneKante();
    }
      
    // **************** Abschnitt fuer die Grafik *****************
    /**
     * Berechnet den zu zeichnenden Graphen anhand der gegebenen Daten.
     */
    protected Shape gibAktuelleFigur()
    {
        GeneralPath graph = new GeneralPath();
        for (int i=0;i<Kante.gibAnzahl();i++){
            graph.append(Kante.gibKante(i), false);
        }
        return graph;
    }
  
    /**
     * Zeichne dieses Objekt mit seinen aktuellen Werten auf den Bildschirm.
     */
    public void zeichne()
    {
        Shape figur = gibAktuelleFigur();
        Leinwand leinwand = Leinwand.gibLeinwand();
        leinwand.zeichne (
          this,           // leinwand kennt das Objekt
          farbe,          // definiert seine Zeichenfarbe
          figur);         // definiert seinen grafischen Aspekt
    }

    /**
     * Zeichne dieses Objekt mit seinen aktuellen Werten auf den Bildschirm.
     */
    public void zeichne(Kante kante)
    {
        Leinwand leinwand = Leinwand.gibLeinwand();
        leinwand.zeichne (
          kante,           // leinwand kennt das Objekt
          farbe,          // definiert seine Zeichenfarbe
          kante);         // definiert seinen grafischen Aspekt
    }

    /**
     * Loesche diesen Graphen vom Bildschirm.
     */
    public void loesche()
    {
        Leinwand leinwand = Leinwand.gibLeinwand();
        leinwand.entferne(this);
    }
}

